/*******************************************************************************
**	Permedia II chipset driver
********************************************************************************
**
 * Permedia2 framebuffer driver.
 * Copyright (c) 1998-1999 Ilario Nardinocchi (nardinoc@CS.UniBO.IT)
 * Driver used for port under KGI license with permission of Ilario 
 * Nardinocchi.
 * Copyright (c) 1999 Andreas Beck - port to KGI
**
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
*/

#include <3dlabs/pm2fb.h>

#define CVPPC_MEMCLOCK                  83000           /* in KHz */
#define CONFIG_FB_PM2_FIFO_DISCONNECT

#if !defined(__LITTLE_ENDIAN) && !defined(__BIG_ENDIAN)
/* #error	"The endianness of the target host has not been defined." */
#warning	"The endianness of the target host has not been defined. Assume __LITTLE_ENDIAN."
#define __LITTLE_ENDIAN 1
#endif

/* FIXME !! convert to KGI style */
#ifdef DEBUG
#define DPRINTK(a,b...) printk("pm2fb: %s: " a, __FUNCTION__ , ## b)
#else
#define DPRINTK(a,b...)
#endif

/*
 * The _DEFINITIVE_ memory mapping/unmapping functions.
 * This is due to the fact that they're changing soooo often...
 */
#define MMAP(a,b)	ioremap((kgiu32 )(a), b)
#define UNMAP(a,b)	iounmap(a)

/*
 * The _DEFINITIVE_ memory i/o barrier functions.
 * This is due to the fact that they're changing soooo often...
 */

/* FIXME FIXME FIXME - we miss that in the KGI definition up to now.
 */

#if 1
#define DEFW()         wmb()
#define DEFR()         rmb()
#define DEFRW()                mb()
#else
#define DEFW()
#define DEFR()
#define DEFRW()
#endif

#ifndef MIN
#define MIN(a,b) ((a)<(b)?(a):(b))
#endif

#ifndef MAX
#define MAX(a,b) ((a)>(b)?(a):(b))
#endif

/*	Driver state is kept in chipset_private.
*/
struct chipset_private
{
	/*	internal stuff
	*/
	mem_vaddr	mem_fb;
	mem_vaddr	mem_regs;	/* memory mapped accel ports	*/

	kgi_uint		frame;
	kgi_uint		split;
	kgi_uint		origin, org_xdot, org_ydot, org_offs;
	kgi_uint		fb_size;

	kgiu32		ptr_old, cur_old;
	kgiu16		ptr_and, cur_and;
	kgiu16		ptr_xor, cur_xor;

	kgiu32 		mem_config;
	kgiu32 		mem_control;
	kgiu32 		boot_address;
	kgiu32 		memclock;	/* memclock (set by the per-board
					   		init routine) */

	/*	exported stuff
	*/
	struct kgi_mmio_region
			mmio_fb;	/* Framebuffer	*/

	/*	administrative stuff
	*/
	struct mem_region	mem_regs_region;
	struct mem_region	mem_fb_region;

	struct kgi_chipset	chipset_info;
};

#define GET_PRIVATE \
	struct chipset_private *private = KGIM_PRIVATE(dpy, chipset)

#define	MEM_FB			(private->mem_fb_region.baseptr)

#define	MEM_FB1			(private->mem_fb)
#define	MEM_REGS		(private->mem_regs)

#define	MEM_FB_REGION		(private->mem_fb_region)
#define MMIO_FB_REGION		(private->mmio_fb)
#define	MEM_REGS_REGION		(private->mem_regs_region)

#define	CHIPSET_INFO		(private->chipset_info)

inline static kgiu32 RD32(mem_vaddr base, kgis32 off) {

	return mem_in32(base+off);
}

inline static void WR32(mem_vaddr base, kgis32 off, kgiu32 v) {

	return mem_out32(v, base+off);
}

inline static kgiu32 pm2_RD(struct chipset_private* private, kgis32 off) {

	return RD32(MEM_REGS, off);
}

inline static void pm2_WR(struct chipset_private* private, kgis32 off, kgiu32 v) {

	WR32(MEM_REGS, off, v);
}

inline static kgiu32 pm2_RDAC_RD(struct chipset_private* p, kgis32 idx) {

	pm2_WR(p, PM2R_RD_PALETTE_WRITE_ADDRESS, idx);
	DEFRW();
	return pm2_RD(p, PM2R_RD_INDEXED_DATA);
}

inline static void pm2_RDAC_WR(struct chipset_private* p, kgis32 idx,
						kgiu32 v) {

	pm2_WR(p, PM2R_RD_PALETTE_WRITE_ADDRESS, idx);
	DEFW();
	pm2_WR(p, PM2R_RD_INDEXED_DATA, v);
}

#ifdef CONFIG_FB_PM2_FIFO_DISCONNECT
#define WAIT_FIFO(p,a)
#else
inline static void WAIT_FIFO(struct chipset_private* p, kgiu32 a) {
	while(pm2_RD(p, PM2R_IN_FIFO_SPACE)<a);
	DEFRW();
}
#endif

